/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "KeyboardEvent.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMWindow.h"
#include "gdom/GdomKeyboardEvent.h"
#include "gdom/GdomUIEvent.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDOMWindow.h"
#include "gdom/PrivateGdomKeyboardEvent.h"
#include "gdom/PrivateGdomUIEvent.h"
#include "gdom/PrivateIntGdomDOMWindow.h"
#include "gdom/PrivateIntGdomKeyboardEvent.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMKeyboardEvent::GDOMKeyboardEvent(PassRefPtr<WebCore::KeyboardEvent> impl)
    : GDOMUIEvent(impl)
{
}

gboolean
gdom_keyboard_event_get_modifier_state (GdomKeyboardEvent *thiz, gchar * key_identifier_arg)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::KeyboardEvent * item = (gdom_keyboard_event_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (key_identifier_arg, 0);
    WebCore::String _g_key_identifier_arg = WebCore::String::fromUTF8(key_identifier_arg);
    gboolean res = item->getModifierState(_g_key_identifier_arg);
    return res;

}

void
gdom_keyboard_event_init_keyboard_event (GdomKeyboardEvent *thiz, gchar * type, gboolean can_bubble, gboolean cancelable, GdomDOMWindow * view, gchar * key_identifier, gulong key_location, gboolean ctrl_key, gboolean alt_key, gboolean shift_key, gboolean meta_key, gboolean alt_graph_key)
{
    g_return_if_fail (thiz);
    WebCore::KeyboardEvent * item = (gdom_keyboard_event_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (type);
    g_return_if_fail (view);
    g_return_if_fail (key_identifier);
    WebCore::String _g_type = WebCore::String::fromUTF8(type);
    WebCore::DOMWindow * _g_view = (gdom_dom_window_get_private(view));
    g_return_if_fail (_g_view);
    WebCore::String _g_key_identifier = WebCore::String::fromUTF8(key_identifier);
    item->initKeyboardEvent(_g_type, can_bubble, cancelable, _g_view, _g_key_identifier, key_location, ctrl_key, alt_key, shift_key, meta_key, alt_graph_key);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "KeyboardEvent.h"
#include <gdom/GdomKeyboardEvent.h>
#include <gdom/PrivateGdomKeyboardEvent.h>
#include <gdom/PrivateIntGdomKeyboardEvent.h>
#include <gdom/GdomKeyboardEventprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomKeyboardEvent, gdom_keyboard_event, GDOM_TYPE_UI_EVENT);

/*
struct _GdomKeyboardEventPrivate {
    class WebCore::KeyboardEvent *item;
};

#define GDOM_KEYBOARD_EVENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_KEYBOARD_EVENT, GdomKeyboardEventPrivate))
*/
G_END_DECLS
WebCore::KeyboardEvent *
gdom_keyboard_event_get_private(GdomKeyboardEvent *request)
{
    GDOMKeyboardEvent *res;
    res = static_cast<GDOMKeyboardEvent*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::KeyboardEvent*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_KEY_IDENTIFIER,
    PROP_KEY_LOCATION,
    PROP_CTRL_KEY,
    PROP_SHIFT_KEY,
    PROP_ALT_KEY,
    PROP_META_KEY,
    PROP_ALT_GRAPH_KEY,
};


static void gdom_keyboard_event_finalize(GObject* object)
{
    /*GdomKeyboardEvent* request = GDOM_KEYBOARD_EVENT(object);*/
    /*GdomKeyboardEventPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_UI_EVENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_keyboard_event_parent_class)->finalize(object);
}

static void gdom_keyboard_event__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomKeyboardEvent *self = GDOM_KEYBOARD_EVENT(object);
    WebCore::KeyboardEvent * gself = (gdom_keyboard_event_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_keyboard_event__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomKeyboardEvent *self = GDOM_KEYBOARD_EVENT(object);
    WebCore::KeyboardEvent * gself = (gdom_keyboard_event_get_private(self));
    switch (prop_id) {
    case PROP_KEY_IDENTIFIER:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->keyIdentifier()));
         break;
    }
    case PROP_KEY_LOCATION:
    {
         g_value_set_ulong (value, gself->keyLocation());
         break;
    }
    case PROP_CTRL_KEY:
    {
         g_value_set_boolean (value, gself->ctrlKey());
         break;
    }
    case PROP_SHIFT_KEY:
    {
         g_value_set_boolean (value, gself->shiftKey());
         break;
    }
    case PROP_ALT_KEY:
    {
         g_value_set_boolean (value, gself->altKey());
         break;
    }
    case PROP_META_KEY:
    {
         g_value_set_boolean (value, gself->metaKey());
         break;
    }
    case PROP_ALT_GRAPH_KEY:
    {
         g_value_set_boolean (value, gself->altGraphKey());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_keyboard_event_class_init(GdomKeyboardEventClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_keyboard_event_finalize;
    g_klass->set_property = gdom_keyboard_event__set_property_fn;
    g_klass->get_property = gdom_keyboard_event__get_property_fn;

    g_object_class_install_property (g_klass, PROP_KEY_IDENTIFIER,
            g_param_spec_string ("key_identifier", /* name */
                "keyboard_event_key_identifier", /* short description */
                "read-only  gchar * KeyboardEvent.key_identifier", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_KEY_LOCATION,
            g_param_spec_ulong ("key_location", /* name */
                "keyboard_event_key_location", /* short description */
                "read-only  gulong KeyboardEvent.key_location", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CTRL_KEY,
            g_param_spec_boolean ("ctrl_key", /* name */
                "keyboard_event_ctrl_key", /* short description */
                "read-only  gboolean KeyboardEvent.ctrl_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SHIFT_KEY,
            g_param_spec_boolean ("shift_key", /* name */
                "keyboard_event_shift_key", /* short description */
                "read-only  gboolean KeyboardEvent.shift_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ALT_KEY,
            g_param_spec_boolean ("alt_key", /* name */
                "keyboard_event_alt_key", /* short description */
                "read-only  gboolean KeyboardEvent.alt_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_META_KEY,
            g_param_spec_boolean ("meta_key", /* name */
                "keyboard_event_meta_key", /* short description */
                "read-only  gboolean KeyboardEvent.meta_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ALT_GRAPH_KEY,
            g_param_spec_boolean ("alt_graph_key", /* name */
                "keyboard_event_alt_graph_key", /* short description */
                "read-only  gboolean KeyboardEvent.alt_graph_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomKeyboardEventPrivate));*/
}

static void gdom_keyboard_event_init(GdomKeyboardEvent* request)
{
    /*GdomKeyboardEventPrivate* priv = GDOM_KEYBOARD_EVENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

